package eu.dnetlib.uoaadmintools.controllers;

import eu.dnetlib.uoaadmintools.dao.CommunityDAO;
import eu.dnetlib.uoaadmintools.dao.DivHelpContentDAO;
import eu.dnetlib.uoaadmintools.dao.DivIdDAO;
import eu.dnetlib.uoaadmintools.dao.PageDAO;
import eu.dnetlib.uoaadmintools.entities.*;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@RestController
@CrossOrigin(origins = "*")
public class DivHelpContentController {
    private final Logger log = Logger.getLogger(this.getClass());

    @Autowired
    DivIdController divIdController;

    @Autowired
    private DivHelpContentDAO divHelpContentDAO;

    @Autowired
    private DivIdDAO divIdDAO;

    @Autowired
    private PageDAO pageDAO;

    @Autowired
    private CommunityDAO communityDAO;

    @RequestMapping(value = "/divhelpcontent", method = RequestMethod.GET)
    public List<DivHelpContentResponse> getDivHelpContents(@RequestParam(required = false) String community,
                                                           @RequestParam(required = false) String page,
                                                           @RequestParam(required = false) String div,
                                                           @RequestParam(required = false) String active) {

        Community _community = communityDAO.findByPid(community);
        String communityId = null;
        if(_community != null) {
            communityId = _community.getId();
        }

        DivId divId = divIdDAO.findByName(div);
        String divIdId = null;
        if(divId != null) {
            divIdId = divId.getId();
        }

        List<DivHelpContentResponse> divHelpContentResponses = new ArrayList<>();

        List<DivHelpContent> divHelpContents = null;
        if(community != null && div != null && active != null) {
            divHelpContents = divHelpContentDAO.findByCommunityAndDivIdAndIsActive(communityId, divIdId, Boolean.parseBoolean(active));
        } else if(community != null && div != null) {
            divHelpContents = divHelpContentDAO.findByCommunityAndDivId(communityId, divIdId);
        } else if(community != null && active != null) {
            divHelpContents = divHelpContentDAO.findByCommunityAndIsActive(communityId, Boolean.parseBoolean(active));
        } else if(div != null && active != null) {
            divHelpContents = divHelpContentDAO.findByDivIdAndIsActive(divIdId, Boolean.parseBoolean(active));
        } else if(community != null) {
            divHelpContents = divHelpContentDAO.findByCommunity(communityId);
        } else if(div != null) {
            divHelpContents = divHelpContentDAO.findByDivId(divIdId);
        } else if(active != null){
            divHelpContents = divHelpContentDAO.findByIsActive(Boolean.parseBoolean(active));
        } else {
            divHelpContents = divHelpContentDAO.findAll();
        }

        for (DivHelpContent divHelpContent : divHelpContents) {
            DivIdResponse divIdResponse = null;
            if(div == null) {
                divId = divIdDAO.findById(divHelpContent.getDivId());
            }
            divIdResponse = divIdController.divIdResponseFromDivId(divId);

            if(community == null) {
                _community = communityDAO.findById(divHelpContent.getCommunity());
            }

            for(Page p : divIdResponse.getPages()) {
                if (page == null || p.getRoute().equals(page)) {
                    DivHelpContentResponse divHelpContentResponse = new DivHelpContentResponse(divHelpContent);
                    divHelpContentResponse.setDivId(divIdResponse);
                    divHelpContentResponse.setCommunity(_community);
                    divHelpContentResponses.add(divHelpContentResponse);
                    break;
                }
            }
        }

        return divHelpContentResponses;
    }

    @RequestMapping(value = "/divhelpcontent/{id}", method = RequestMethod.GET)
    public DivHelpContent getDivHelpContent(@PathVariable(value = "id") String id) {
        return divHelpContentDAO.findById(id);
    }

    @RequestMapping(value = "/divhelpcontent", method = RequestMethod.POST)
    public DivHelpContent insertOrUpdateDivHelpContent(@RequestBody DivHelpContent divHelpContent) {
        return divHelpContentDAO.save(divHelpContent);
    }

    @RequestMapping(value = "/divhelpcontent/{id}", method = RequestMethod.DELETE)
    public void deleteDivHelpContent(@PathVariable(value = "id") String id) {
        divHelpContentDAO.delete(id);
    }

    @RequestMapping(value = "/divhelpcontent/delete", method = RequestMethod.POST)
    public Boolean deleteDivHelpContents(@RequestBody List<String> divHelpContents) throws Exception {
        for (String id: divHelpContents) {
            divHelpContentDAO.delete(id);
        }
        return true;
    }

    @RequestMapping(value = "/divhelpcontent/toggle", method = RequestMethod.POST)
    public List<String> toggleDivHelpContent(@RequestBody List<String> divHelpContents, @RequestParam String status) throws Exception {
        for (String id: divHelpContents) {
            log.debug("Id of divHelpContent: "+id);
            DivHelpContent divHelpContent = divHelpContentDAO.findById(id);
            divHelpContent.setIsActive(Boolean.parseBoolean(status));
            divHelpContentDAO.save(divHelpContent);
        }
        return divHelpContents;
    }

    public void addDivHelpContentsInCommunity(String pid, String communityId, String divIdName) {
        //String organizations_class_content = "<div> <p>Here you can write more details about the organizations related to your community.</p> </div>";
        String link_context_form_content = "<div> <div><span class=\"uk-text-bold\"><span uk-icon=\"icon: info\">&nbsp;</span> Information:</span> Select a research community and/or a category and search for a community concept, or browse to the community tree through the categories</div> </div>";
        String link_project_form_content = "<div> <div><span class=\"uk-text-bold\"><span uk-icon=\"icon: info\">&nbsp;</span> Information:</span> Search for projects using project name or grant id. Limit results filtering by funder.</div> </div>";
        String link_result_form_content = "<div> <div><span class=\"uk-text-bold\"><span uk-icon=\"icon: info\">&nbsp;</span> Information:</span></div> Search for research results in OpenAIRE information space, Datacite, CrossRef or ORCID. <div class=\"uk-text-small\">Use keywords, DOI (more than one - space separated), author&#39;s ORCID</div> </div>";
        String link_result_bulk_content = "<div> <div><span class=\"uk-text-bold\"><span uk-icon=\"icon: info\">&nbsp;</span> Information:</span> Upload a csv file containing a list of DOIs. For each DOI found in the file, metadata will be fetched from CrossRef or Datacite and will be added to your selected research results.</div> <div class=\"uk-margin-top uk-text-small\"><span class=\"uk-text-bold\">CSV format:</span> <ul class=\"uk-list\"> <li>The format of CSV file should be &quot;DOI&quot;,&quot;ACCESS_MODE&quot;,&quot;DATE&quot;.</li> <li>The value &quot;DOI&quot; is required</li> <li>Access mode column should have values: &quot;OPEN&quot;,&quot;CLOSED&quot; or &quot;EMBARGO&quot;.</li> <li>Date column valid format is YYYY-MM-DD and is required when access mode has value EMBARGO.</li> <li>In case access mode is not available default value is &quot;OPEN&quot;.</li> </ul> </div> </div>";
        String link_metadata_content = "<div> <div><span class=\"uk-text-bold\"><span uk-icon=\"icon: info\">&nbsp;</span> Information:</span> Manage access mode &amp; type of selected research results. For OpenAIRE this functionality isn&#39;t available.</div> </div>";

        List<DivId> divIds = divIdController.getDivIds(null, divIdName, null);

        for( DivId div : divIds ) {
            if (div != null && (
                (div.getOpenaire() && pid.equals("openaire")) ||
                (div.getConnect() && pid.equals("connect")) ||
                (div.getCommunities() && !pid.equals("openaire") && !pid.equals("connect"))
            )) {
                /*
                if (div.getName().equals("organizations")) {
                    this.insertOrUpdateDivHelpContent(new DivHelpContent(div.getId(), communityId, organizations_class_content, false));
                } else
                */
                if (div.getName().equals("link-context-form")) {
                    this.insertOrUpdateDivHelpContent(new DivHelpContent(div.getId(), communityId, link_context_form_content, false));
                } else if (div.getName().equals("link-project-form")) {
                    this.insertOrUpdateDivHelpContent(new DivHelpContent(div.getId(), communityId, link_project_form_content, false));
                } else if (div.getName().equals("link-result-form")) {
                    this.insertOrUpdateDivHelpContent(new DivHelpContent(div.getId(), communityId, link_result_form_content, false));
                } else if (div.getName().equals("link-result-bulk")) {
                    this.insertOrUpdateDivHelpContent(new DivHelpContent(div.getId(), communityId, link_result_bulk_content, false));
                } else if (div.getName().equals("link-metadata")) {
                    this.insertOrUpdateDivHelpContent(new DivHelpContent(div.getId(), communityId, link_metadata_content, false));
                }
            }
        }
    }

}
