package eu.dnetlib.uoaadmintools.controllers;

import eu.dnetlib.uoaadmintools.dao.CommunityDAO;
import eu.dnetlib.uoaadmintools.dao.DivIdDAO;
import eu.dnetlib.uoaadmintools.entities.*;
import eu.dnetlib.uoaadmintools.dao.PageDAO;

import org.apache.log4j.Logger;
import org.springframework.web.bind.annotation.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.*;

@RestController
@CrossOrigin(origins = "*")
public class PageController {
    private final Logger log = Logger.getLogger(this.getClass());

    @Autowired
    private PageDAO pageDAO;

    @Autowired
    private CommunityDAO communityDAO;

    @Autowired
    private DivIdDAO divIdDAO;

    @Autowired
    private PageHelpContentController pageHelpContentController;

    @Autowired
    private DivHelpContentController divHelpContentController;

    @Autowired
    private DivIdController divIdController;

    @Autowired
    private EntityController entityController;

    @RequestMapping(value = "/pageFull", method = RequestMethod.GET)
    public List<CommunityPage> getPagesFull(@RequestParam(value="pid", required=false) String pid,
                                            @RequestParam(value="page_route", required=false) String page_route) {

        List<Page> pages = this.getAllPages(pid, page_route, null);

        List<CommunityPage> communityPages = new ArrayList<>();
        for(Page page : pages) {
            CommunityPage communityPage = new CommunityPage(page);
            List<Entity> entities = new ArrayList<>();
            for(String entityId : page.getEntities()) {
                entities.add(entityController.getEntity(entityId));
            }
            communityPage.setEntities(entities);

            communityPages.add(communityPage);
        }

        return communityPages;
    }

    @RequestMapping(value = "/page", method = RequestMethod.GET)
    public List<Page> getAllPages(@RequestParam(value="pid", required=false) String pid,
                                  @RequestParam(value="page_route", required=false) String page_route,
                                  @RequestParam(value="with_positions", required=false) String with_positions) {
        List<Page> pages;
        if (pid != null) {
            if (pid.equals("openaire")) {
                if (page_route != null) {
                    pages = new ArrayList<Page>();
                    pages.add(pageDAO.findByOpenaireAndRoute(true, page_route));
                } else {
                    pages = pageDAO.findByOpenaire(true);
                }
            } else if (pid.equals("connect")) {
                if (page_route != null) {
                    pages = new ArrayList<Page>();
                    pages.add(pageDAO.findByConnectAndRoute(true, page_route));
                } else {
                    pages = pageDAO.findByConnect(true);
                }
            } else {
                if (page_route != null) {
                    pages = new ArrayList<Page>();
                    pages.add(pageDAO.findByCommunitiesAndRoute(true, page_route));
                } else {
                    pages = pageDAO.findByCommunities(true);
                }
            }
        } else if (page_route != null) {
            pages = new ArrayList<Page>();
            pages.add(pageDAO.findByRoute(page_route));
        } else {
            pages = pageDAO.findAll();
        }

        if (with_positions != null) {
            boolean at_least_one_position = Boolean.parseBoolean(with_positions);

            Iterator<Page> iteratorPages = pages.iterator();
            while(iteratorPages.hasNext()) {
                Page page = iteratorPages.next();

                if(at_least_one_position) {
                    if(!page.getTop() && !page.getBottom() && !page.getLeft() && !page.getRight()) {
                        iteratorPages.remove();
                    }
                } else {
                    if(page.getTop() || page.getBottom() || page.getLeft() || page.getRight()) {
                        iteratorPages.remove();
                    }
                }
            }
        }
        pages.sort(Comparator.comparing(Page::getName));
        return pages;
    }

    @RequestMapping(value = "/page", method = RequestMethod.DELETE)
    public void deleteAllPages() {
        pageDAO.deleteAll();
    }

    @RequestMapping(value = "/page/update", method = RequestMethod.POST)
    public CommunityPage updatePage(@RequestBody CommunityPage communityPage) {
        this.deletePageHelpContentsForPositionsIfDisabled(communityPage);
        Page page = this.getPageByCommunityPage(communityPage);
        pageDAO.save(page);
        return communityPage;
    }

    @RequestMapping(value = "/page/save", method = RequestMethod.POST)
    public CommunityPage insertPage(@RequestBody CommunityPage communityPage) {
        Page page = this.getPageByCommunityPage(communityPage);
        Page savedPage = pageDAO.save(page);
        communityPage.setId(savedPage.getId());

        // add page in communities
        List<Community> communities = communityDAO.findAll();
        for( Community community : communities ) {
            Map<String, Boolean> pages = community.getPages();
            pages.put(page.getId(), true);
            community.setPages(pages);
            communityDAO.save(community);
        }

        return communityPage;
    }

    private Page getPageByCommunityPage(CommunityPage communityPage) {
        Page page = new Page();
        page.setId(communityPage.getId());
        page.setRoute(communityPage.getRoute());
        page.setName(communityPage.getName());
        page.setType(communityPage.getType());
        page.setConnect(communityPage.getConnect());
        page.setCommunities(communityPage.getCommunities());
        page.setOpenaire(communityPage.getOpenaire());
        page.setTop(communityPage.getTop());
        page.setBottom(communityPage.getBottom());
        page.setLeft(communityPage.getLeft());
        page.setRight(communityPage.getRight());

        List<Entity> fullEntities = communityPage.getEntities();
        List<String> entities = new ArrayList<String>();
        for(Entity entity : fullEntities) {
            entities.add(entity.getId());
        }
        page.setEntities(entities);

        return page;
    }

    private void deletePageHelpContentsForPositionsIfDisabled(CommunityPage communityPage) {

        if(!communityPage.getTop()) {
            // delete page contents with position "top" related to this page from all communities
            List<PageHelpContentResponse> pageHelpContentResponses = pageHelpContentController.getPageHelpContents(null, communityPage.getRoute(), "top", null, null);
            for(PageHelpContentResponse pageHelpContentResponse : pageHelpContentResponses) {
                pageHelpContentController.deletePageHelpContent(pageHelpContentResponse.getId());
            }
        }

        if(!communityPage.getBottom()) {
            // delete page contents with position "bottom" related to this page from all communities
            List<PageHelpContentResponse> pageHelpContentResponses = pageHelpContentController.getPageHelpContents(null, communityPage.getRoute(), "bottom", null, null);
            for(PageHelpContentResponse pageHelpContentResponse : pageHelpContentResponses) {
                pageHelpContentController.deletePageHelpContent(pageHelpContentResponse.getId());
            }
        }

        if(!communityPage.getLeft()) {
            // delete page contents with position "left" related to this page from all communities
            List<PageHelpContentResponse> pageHelpContentResponses = pageHelpContentController.getPageHelpContents(null, communityPage.getRoute(), "left", null, null);
            for(PageHelpContentResponse pageHelpContentResponse : pageHelpContentResponses) {
                pageHelpContentController.deletePageHelpContent(pageHelpContentResponse.getId());
            }
        }

        if(!communityPage.getRight()) {
            // delete page contents with position "right" related to this page from all communities
            List<PageHelpContentResponse> pageHelpContentResponses = pageHelpContentController.getPageHelpContents(null, communityPage.getRoute(), "right", null, null);
            for(PageHelpContentResponse pageHelpContentResponse : pageHelpContentResponses) {
                pageHelpContentController.deletePageHelpContent(pageHelpContentResponse.getId());
            }
        }
    }

    @RequestMapping(value = "/page/delete", method = RequestMethod.POST)
    public Boolean deletePages(@RequestBody List<String> pages) throws Exception {
        for (String id: pages) {
            pageDAO.delete(id);

            // delete divIds related only to this page from all communities, otherwise remove this page from divIds
            List<DivId> divIds = divIdController.getDivIds(id, null, null);
            for(DivId divId : divIds) {
                if(divId.getPages().size() == 1) {
                    divIdController.deleteDivId(divId.getId());

                    // delete div contents related to this page from all communities
                    List<DivHelpContentResponse> divHelpContentResponses = divHelpContentController.getDivHelpContents(null, id, divId.getId(), null);
                    for (DivHelpContentResponse divHelpContentResponse : divHelpContentResponses) {
                        divHelpContentController.deleteDivHelpContent(divHelpContentResponse.getId());
                    }
                } else {
                    List<String> divIdPages = divId.getPages();
                    divIdPages.remove(id);
                    divId.setPages(divIdPages);
                    divIdDAO.save(divId);
                }
            }


            // delete page contents related to this page from all communities
            List<PageHelpContentResponse> pageHelpContentResponses = pageHelpContentController.getPageHelpContents(null, id, null, null, null);
            for(PageHelpContentResponse pageHelpContentResponse : pageHelpContentResponses) {
                pageHelpContentController.deletePageHelpContent(pageHelpContentResponse.getId());
            }

            // delete page from communities
            List<Community> communities = communityDAO.findAll();
            for( Community community : communities ) {
                Map<String, Boolean> communityPages = community.getPages();
                communityPages.remove(id);
                community.setPages(communityPages);
                communityDAO.save(community);
            }
        }

        return true;
    }

    @RequestMapping(value = "/page/{id}", method = RequestMethod.GET)
    public Page getPage(@PathVariable(value = "id") String id) {
        return pageDAO.findById(id);
    }

    @RequestMapping(value = "/page/{id}", method = RequestMethod.DELETE)
    public void deletePage(@PathVariable(value = "id") String id) {
        pageDAO.delete(id);
    }

    @RequestMapping(value = "/page/{id}/entity", method = RequestMethod.GET)
    public List<String> getPageEntities(@PathVariable(value = "id") String id) {
        return pageDAO.findById(id).getEntities();
    }

    @RequestMapping(value = "page/{id}/entity/toggle", method = RequestMethod.POST)
    public Page togglePageEntity(@PathVariable(value = "id") String id, @RequestParam String entityId, @RequestParam String status) throws Exception {
        log.debug("Toggle entity : "+entityId +" of page: "+id+" to "+status);
        Page page = pageDAO.findById(id);
        List<String> entities = page.getEntities();
        if(Boolean.parseBoolean(status) && !entities.contains(entityId)) {
            entities.add(entityId);
        } else if (!Boolean.parseBoolean(status)) {
            entities.remove(entityId);
        }
        page.setEntities(entities);
        return pageDAO.save(page);
    }
}

